﻿def processStep1(step):
    clr.AddReference("Ansys.Study.UI")
    from Ansys.Study.UI.Utils import ContextEventsHelper

    # SET FIRST UPDATE FLAG
    if step.Attributes["isFirstUpdate"] == None: step.Attributes["isFirstUpdate"] = True

    # CHECK IF NOT FIRST TIME THROUGH
    if step.Attributes["isFirstUpdate"]==False:
        system1 = GetSystem(Name="Study")
        study1 = system1.GetContainer(ComponentName="Study")

        geometryModelingComponent1 = system1.GetComponent(Name="Geometry Modeling")
        modeling1 = geometryModelingComponent1.GetTaskObject()
        modeling1.SetImportPath(FilePath=step.Properties["geometry"].ValueString)
        geometryModelingComponent1.Update(AllDependencies=True)
        
        ContextEventsHelper.ChangeModelSelectionContextTo(modeling1)
        
        return True

    # SETUP WORKFLOW IF FIRST TIME THROUGH
    system1 = GetSystem(Name="Study")
    geometryModelingComponent1 = Study.CreateTask(
        Type="Geometry Modeling",
        System=system1)
    modeling1 = geometryModelingComponent1.GetTaskObject()
    modeling1.SetImportPath(FilePath=step.Properties["geometry"].ValueString)
    meshingComponent1 = Study.CreateTask(
        Type="Meshing",
        System=system1,
        Input=geometryModelingComponent1)
    meshing1 = meshingComponent1.GetTaskObject()
    meshing1.EngineeringIntent = "StructuralOrThermalOrElectricConduction"
    meshingComponent1.Refresh()
    physicsDefinitionComponent1 = Study.CreateTask(
        Type="Physics Definition",
        System=system1,
        Input=meshingComponent1)
    physicsDefinition1 = physicsDefinitionComponent1.GetTaskObject()
    physicsDefinition1.CalculationType = "Static"
    study1 = system1.GetContainer(ComponentName="Study")
    solverSettings1 = study1.CreateEntity(
        Type="SolverSettings",
        Association=physicsDefinition1)
    solvePhysicsComponent1 = Study.CreateTask(
        Type="Solve Physics",
        System=system1,
        Input=physicsDefinitionComponent1)
    solvePhysicsComponent1.Refresh()
    physicsSolutionGroup1 = Study.CreateGroup(Name="Physics Solution")
    physicsSolutionGroup1.Add(Component=physicsDefinitionComponent1)
    physicsSolutionGroup1.Add(Component=solvePhysicsComponent1)
    physicsSolutionGroup1.DisplayText = "Buckling"
    physicsRegion1 = study1.CreateEntity(
        Type="PhysicsRegion",
        Association=physicsDefinition1)
    physicsRegion1.PhysicsType = "Structural"
    physicsRegion1.Location = "AllBodies()"
    physicsRegion1.DisplayText = "LinearBuckling"
    materialAssignment1 = study1.CreateEntity(
        Type="MaterialAssignment",
        Association=physicsDefinition1)
    material1 = study1.CreateEntity(
        Type="Material",
        Association=physicsDefinition1)
    material1.ImportEngineeringData(Name="Structural Steel")
    material1.DisplayText = "Material"
    materialAssignment1.Material = material1
    materialAssignment1.Location = [physicsRegion1]
    interfaceGenerator1 = study1.CreateEntity(
        Type="InterfaceGenerator",
        Association=physicsDefinition1)
    transcript1 = study1.CreateEntity(
        Type="Transcript",
        Association=physicsDefinition1)
    transcript1.DisplayText = "Structural Output 1"
    resultsEvaluationComponent1 = Study.CreateTask(
        Type="Results Evaluation",
        System=system1,
        Input=solvePhysicsComponent1)
    AddSourceToComponentInSystem(
        SourceComponent=physicsDefinitionComponent1,
        TargetComponent=resultsEvaluationComponent1)
    resultsEvaluationComponent1.Refresh()
    results1 = resultsEvaluationComponent1.GetTaskObject()
    contourResult1 = study1.CreateEntity(
        Type="ContourResult",
        Association=results1)
    contourResult1.Variable = "Displacement.mag"
    contourResult1.DisplayText = "Buckling Mode 1"    
    geometryModelingComponent1.Update(AllDependencies=True)
    interfaceGenerator1.GenerateInteractions()
    support1 = study1.CreateEntity(
        Type="Support",
        Association=physicsDefinition1)
    force1 = study1.CreateEntity(
        Type="Force",
        Association=physicsDefinition1)
    force1.Vector.VectorByMagnitudeAndDirection.Magnitude = "1 [N]"
    force1.Vector.VectorByMagnitudeAndDirection.ReverseDirection = "True"
    userCommands1 = study1.CreateEntity(
        Type="UserCommands",
        Association=physicsDefinition1)
    userCommands1.AddCodeSnippet(
        SectionPath="Solve/Loadcase/JustBeforeSolveSection",
        Commands="""solve
    finish
    /assign,rstp,file,rst
    /solu
    antype,,restart,,,perturb
    perturb,buckle,,CURRENT,ALLKEEP 
    solve,elform
    bucopt,lanb,1,,,center
    outres,erase
    outres,all,all""")
    userCommands1.AddCodeSnippet(
        SectionPath="PostProcessingSection",
        Commands=r"""/OUT,..\..\..\..\user_files\myloadfactors,txt
    *GET,FREQ,MODE,1,FREQ
    /output""")

    # CREATE NAMED EXPRESSION TO STORE BUCKLING LOAD
    scope1 = Parameters.GetScope(Name="Scope")
    namedExpression1 = Parameters.CreateNamedExpression(
        Name="NamedExpression1",
        Scope=scope1)
    Parameters.RenameNamedExpression(
        NamedExpression=namedExpression1,
        NewName="BucklingLoad")
    Parameters.SetNamedExpression(
        NamedExpression=namedExpression1,
        Expression="0[N]")
    
    
    # DISPLAY MODEL AT END OF UPDATE
    ContextEventsHelper.ChangeModelSelectionContextTo(modeling1)

    step.Attributes["isFirstUpdate"] = False


def processStep2(step):

  #  try:
    clr.AddReference("Ansys.Study.UI")
    from Ansys.Study.UI.Utils import ContextEventsHelper

    # SET FIRST UPDATE FLAG
    if step.Attributes["isFirstUpdate"] == None: step.Attributes["isFirstUpdate"] = True

    system1 = GetSystem(Name="Study")
    study1 = system1.GetContainer(ComponentName="Study")
    support1 = study1.GetSupport(Name="Support 1")
    support1.Location = step.Properties["support"].Value.LocationSet
    force1 = study1.GetForce(Name="Force 1")
    force1.Location = step.Properties["force"].Value.LocationSet

    material1 = study1.GetMaterial(Name="Material 1")    
    # material1.Solid.IsotropicElasticity.Magnitude.YoungsModulus = step.Properties["YoungsModulus"].ValueString
    # material1.Solid.IsotropicElasticity.Magnitude.PoissonsRatio = step.Properties["PoissonRatio"].ValueString

    
    youngsModulus = step.Properties["YoungsModulus"].Value.ToString(System.Globalization.CultureInfo.InvariantCulture)   
    youngsModulus_Unit=step.Properties["YoungsModulus"].UnitString     
    poissonsRatio = step.Properties["PoissonRatio"].Value.ToString(System.Globalization.CultureInfo.InvariantCulture)            
    material1.Solid.IsotropicElasticity.Magnitude.YoungsModulus = youngsModulus + " [" + str(youngsModulus_Unit) + "]"   
    material1.Solid.IsotropicElasticity.Magnitude.PoissonsRatio = poissonsRatio 

    # SOLVE
    resultsEvaluationComponent1 = system1.GetComponent(Name="Results Evaluation")
    resultsEvaluationComponent1.Update(AllDependencies=True)
    study1 = system1.GetContainer(ComponentName="Study")
    contourResult1 = GetLastCreatedObjectByUserName ("Buckling Mode 1")
    #contourResult1 = study1.GetContourResult(Name="ContourResult 1")
    contourResult1.Evaluate()

    # get loadfactor from a file
    userDir = GetProjectFile().split(".wbpj")[0] + "_files"
    file = userDir + "\\user_files\\" + "myloadfactors"
    frequencyFile = file +".txt"       

    # GET THE LOAD FACTORS
    f = open(frequencyFile, 'r')
    text = f.readline()
    text = f.readline()
    i = text.index("U")
    i = i+5
    force1 = text[i:i+10] + " [N]"

    # find the results step and set the quantitative result
    s = step.NextStep
    s.Properties["bucklingLoad"].Value = force1
    
    # store buckling load in named expression
    scope1 = Parameters.GetScope(Name="Scope")    
    namedExpression1 = Parameters.GetNamedExpression(
    Name="BucklingLoad",
    Scope=scope1)
    Parameters.SetNamedExpression(
    NamedExpression=namedExpression1,
    Expression=force1)

    # SET PLOT TO RESULT
    contourResult1 = study1.GetContourResult(Name="ContourResult 1")
    ContextEventsHelper.ChangeModelSelectionContextTo(contourResult1)
    
    # SET FIRST UPDATE FLAG
    step.Attributes["isFirstUpdate"] = False
# except BaseException as e:
# 	ExtAPI.Log.WriteMessage("Error in processStep2: " + e.ToString())
#     raise e

def onrefreshStep2(step):
    # Autogenerated function
    clr.AddReference("Ansys.Study.UI")
    from Ansys.Study.UI.Utils import ContextEventsHelper

    system1 = GetSystem(Name="Study")
    study1 = system1.GetContainer(ComponentName="Study")
    try:
        force1 = study1.GetForce(Name="Force 1")
        ContextEventsHelper.ChangeModelSelectionContextTo(force1)
    except:
        force1 = None

    pass
         
def processStep3(step):
    # Autogenerated function
    
    clr.AddReference("Ans.UI.Toolkit")
    clr.AddReference("Ans.UI.Toolkit.Base")
    clr.AddReference("Ansys.Study.UI")
    import Ansys.Study.UI
    from Ansys.UI.Toolkit import *
    
    # SET FIRST UPDATE FLAG
    if step.Attributes["isFirstUpdate"] == None: step.Attributes["isFirstUpdate"] = True   

    # generate report
    userDir = GetProjectFile().split(".wbpj")[0] + "_files"
    file = userDir + "\\user_files\\" + "eulerbuckling.html"
    ExportReport(FilePath=file)   
    MessageBox.Show("Report written to user_files: "+file)

    step.Attributes["isFirstUpdate"] = False

    pass

def onrefreshStep3(step):
    # Autogenerated function

    clr.AddReference("Ansys.Study.UI")
    from Ansys.Study.UI.Utils import ContextEventsHelper

    system1 = GetSystem(Name="Study")
    study1 = system1.GetContainer(ComponentName="Study")
    try:
        contourResult1 = study1.GetContourResult(Name="ContourResult 1")
        ContextEventsHelper.ChangeModelSelectionContextTo(contourResult1)
    except:
        contourResult1 = None
        
    pass

# helper functions

def GetStudyObjectsByType(searchType = None):
    # get the list of objects of specified type
    foundObjects=[]
    system1 = GetSystem(Name="Study")
    study1 = system1.GetContainer(ComponentName="Study")
    allObjects = GetDataEntities(Container=study1)
    if searchType==None:
        return allObjects
    else:
        for studyObject in allObjects:
            if studyObject.Type==searchType:
                foundObjects.append(studyObject)
    return foundObjects
    
def GetLastCreatedObjectByUserName(searchName):
    # get the last object created with specified user name
    foundObject = None
    for studyObject in GetStudyObjectsByType(None):
        if studyObject.DisplayText==searchName:
            foundObject = studyObject
    return foundObject      

